require "opera-mobile-store-sdk/version"
require 'active_support/all'

module Opera

  module MobileStore
    extend ActiveSupport::Autoload
    autoload :Product
    autoload :ProductLocalization
    autoload :ProductImage
    autoload :Category

    autoload :Compatibility

    autoload :Build
    autoload :BuildFile
    autoload :Developer

    autoload :DeviceModel
    autoload :DevicePlatformFamily
    autoload :DevicePlatform

    autoload :PaymentInfo

    autoload :InspectableAttributes
  end

  module MobileStoreSDK
    extend ActiveSupport::Autoload

    autoload :APIAccessible
    autoload :APIObjectList
    autoload :Config
    autoload :Errors
    autoload :IdentityMapable

    def self.config
      @config ||= Opera::MobileStoreSDK::Config.new
    end

    def self.logger
      config.logger
    end

    def self.html_entities
      @html_entities ||= begin
        require 'htmlentities'
        HTMLEntities.new
      end
    end

    def self.connection
      @connection ||= Faraday.new(url: Opera::MobileStoreSDK.config.api_host) do |faraday|

        faraday.response :logger                  # log requests to STDOUT

        faraday.use MobileStoreSDK::FaradayMiddleware::RequiredResponseFormat

        faraday.use MobileStoreSDK::FaradayMiddleware::Authentication,
                    MobileStoreSDK.config.username,
                    MobileStoreSDK.config.password,
                    authentication: MobileStoreSDK.config.authentication

        faraday.use MobileStoreSDK::FaradayMiddleware::ResponseParser

        faraday.use MobileStoreSDK::FaradayMiddleware::SDKBenchmark

        if defined?(Patron)
          faraday.adapter :patron
        else
          faraday.adapter Faraday.default_adapter  # make requests with Net::HTTP
        end
      end
    end

  end

end
