require "active_model"

module Opera::MobileStore
  class DeviceModel

    include ActiveModel::Model
    include ActiveModel::Serialization

    include Opera::MobileStore::InspectableAttributes

    include Opera::MobileStoreSDK::IdentityMapable

    include Opera::MobileStoreSDK::APIAccessible

    # All attributes are Read-Only...
    attr_accessor :id, :code, :name

    def attributes
      [:id, :code, :name].inject({}) do |hash, field_name|
        field_value = self.public_send field_name
        hash[field_name.to_s] = field_value unless field_value.nil?
        hash
      end
    end

    self.singleton_class.send :alias_method, :deserialize, :new

    def self.build_from_nokogiri_node(node)
      data = {
        id:   node.xpath("number(@id)").to_i,
        code: node.xpath("string(@code)").strip,
        name: node.text.strip
      }.select { |key, val| val.present? }

      self.new data
    end

  end
end
