require "active_model"

module Opera::MobileStore
  class ProductImage

    include ActiveModel::Model
    include ActiveModel::Serialization

    include Opera::MobileStore::InspectableAttributes

    # All attributes are Read-Only...
    attr_accessor :type, :width, :height, :url

    def attributes
      [:type, :width, :height, :url].inject({}) do |hash, field_name|
        field_value = self.public_send field_name
        hash[field_name.to_s] = field_value if field_value.present?
        hash
      end
    end

    def self.build_from_nokogiri_node(node)

      data = {
        type: node.name,
        url: node.text.strip
      }

      # Extract width + height data:
      width = node.xpath("string(@width)")
      height = node.xpath("string(@height)")
      data[:width] = width.to_i if width.present?
      data[:height] = width.to_i if height.present?

      self.new data
    end

    self.singleton_class.send :alias_method, :deserialize, :new

  end
end
