module Opera
  module MobileStoreSDK
    module IdentityMapable

      extend ActiveSupport::Concern

      # Registers itself in the identity map:
      def initialize(params={})
        super

        # Upsert in the identity map:
        if self.class.identity_map.key? self.id
          params.each do |field_name, field_value|
            self.class.identity_map[self.id].public_send(
              "#{field_name}=", field_value
            )
          end
        else
          self.class.identity_map[self.id] = self
        end
      end

      module ClassMethods

        def identity_map
          @_identity_map ||= {}
        end

        def find(given_id, hint_options = {})
          if identity_map.key?(given_id)
            identity_map[given_id]
          else
            raise "Not Implemented"
          end
        end

        def identity_mapped?(object_id)
          identity_map.key? object_id
        end

      end

    end
  end
end
